'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.parse = parse;

var _babylon = require('babylon');

var babylon = _interopRequireWildcard(_babylon);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function parse(text) {
    var source = babylon.parse(text, { sourceType: 'module' });
    var body = source.program.body;

    var lines = text.split('\n');

    var imports = getImports(body, lines);
    var declaration = getDeclaration(body, text);

    if (!declaration) {
        return false;
    }

    var newParts = declaration.parts.filter(function (part) {
        return !imports.find(function (imp) {
            return imp.identifiers.indexOf(part.identifier) > -1;
        });
    }).map(function (part) {
        part.async = part.identifier.toLowerCase().indexOf('async') > -1;
        return part;
    });

    return {
        imports: imports,
        newParts: newParts,
        parts: declaration.parts,
        type: declaration.type
    };
}

function getImports(body, lines) {
    return body.filter(function (node) {
        return node.type === 'ImportDeclaration';
    }).map(function (node) {
        var identifiers = node.specifiers.filter(function (obj) {
            return obj.local.type === 'Identifier';
        }).map(function (identifier) {
            return identifier.local.name;
        });
        var path = node.source.value;
        var start = node.loc.start.line - 1;
        var end = node.loc.end.line;

        return {
            lines: lines.slice(start, end),
            loc: {
                start: start,
                end: end
            },
            identifiers: identifiers,
            path: path
        };
    });
}

function recursiveFindSignals(body) {
    if (body.forEach) {
        for (var i = 0; i < body.length; i++) {
            var result = recursiveFindSignals(body[i]);
            if (result) {
                return result;
            }
        }
    }

    if (body.type === 'ObjectProperty') {
        if (body.key.name === 'signals') {
            return body.value;
        }
    }

    if (body.type === 'ObjectExpression') {
        return recursiveFindSignals(body.properties);
    }

    if (body.type === 'ReturnStatement' && (body.argument.type === 'ArrowFunctionExpression' || body.argument.type === 'FunctionExpression' || body.argument.type === 'ObjectExpression')) {
        return recursiveFindSignals(body.argument);
    }

    if (body.declaration && body.declaration.body) {
        return recursiveFindSignals(body.declaration.body);
    }

    if (body.body) {
        return recursiveFindSignals(body.body);
    }

    return false;
}

function getDeclaration(body, text) {
    if (text.indexOf('signals') > -1) {
        var node = recursiveFindSignals(body);
        if (node) {
            var elements = node.properties;

            var parts = parseCollection(elements);

            return { parts: parts, type: 'ModuleFile' };
        }
    }

    var exportDefaultDeclaration = body.find(function (node) {
        return node.type === 'ExportDefaultDeclaration';
    });
    if (!exportDefaultDeclaration) {
        return false;
    }

    var signalDeclaration = exportDefaultDeclaration.declaration;
    if (signalDeclaration.type === 'ArrayExpression') {
        var _elements = signalDeclaration.elements;
        var _parts = parseCollection(_elements);
        var uniqueParts = _parts.reduce(function (unique, part) {
            var found = unique.find(function (item) {
                return item.identifier === part.identifier;
            });
            if (found) {
                if (found.argumentCount !== undefined && part.argumentCount !== undefined) {
                    found.argumentCount = Math.max(found.argumentCount, part.argumentCount);
                }
            } else {
                unique.push(part);
            }
            return unique;
        }, []);

        return { parts: uniqueParts, type: 'SignalOrChainFile' };
    }

    if (signalDeclaration.type === 'ObjectExpression') {
        var _elements2 = signalDeclaration.properties;
        var _parts2 = parseCollection(_elements2);

        return { parts: _parts2, type: 'SignalDefinitionsFile' };
    }

    return false;
}

function parseCollection(collection) {
    var foundParts = [];
    var lastPart = null;
    var _iteratorNormalCompletion = true;
    var _didIteratorError = false;
    var _iteratorError = undefined;

    try {
        for (var _iterator = collection[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
            var element = _step.value;

            var part = {};
            if (element.type === 'CallExpression') {
                part.type = 'actionFactory';
                part.identifier = element.callee.name;
                part.argumentCount = element.arguments.length;

                var arrays = element.arguments.filter(function (a) {
                    return a.type === 'ArrayExpression';
                });

                if (arrays && arrays.length) {
                    var _subParts = arrays.reduce(function (all, current) {
                        console.log(current.elements);
                        return all.concat.apply(all, _toConsumableArray(parseCollection(current.elements)));
                    }, []);
                    foundParts.push.apply(foundParts, [part].concat(_toConsumableArray(_subParts)));
                    continue;
                }

                var subParts = element.arguments.reduce(function (all, prop) {
                    if (prop.type === 'TaggedTemplateExpression') {
                        return all.concat({
                            type: 'tag',
                            identifier: prop.tag.name
                        });
                    }
                    return all;
                }, []);
                foundParts.push.apply(foundParts, _toConsumableArray(subParts));
            } else if (element.type === 'ObjectExpression') {
                if (lastPart) {
                    lastPart.outputs = element.properties.map(function (prop) {
                        return prop.key.name;
                    });
                }

                var _subParts2 = element.properties.reduce(function (all, prop) {
                    if (prop.value.type === 'Identifier') {
                        return all.concat({
                            type: 'chain',
                            identifier: prop.value.name
                        });
                    }
                    return all.concat(parseCollection([prop.value]));
                }, []);
                foundParts.push.apply(foundParts, _toConsumableArray(_subParts2));

                continue;
            } else if (element.type === 'Identifier') {
                part.type = 'action';
                part.identifier = element.name;
            } else if (element.type === 'SpreadElement') {
                if (element.argument.type === 'CallExpression') {
                    part.identifier = element.argument.callee.name;
                    part.type = 'chainFactory';

                    foundParts.push(part);
                    element.argument.arguments.map(function (item) {
                        var subParts = parseCollection(item.elements);
                        foundParts.push.apply(foundParts, _toConsumableArray(subParts));
                    });

                    continue;
                } else {
                    part.type = 'chain';
                    part.identifier = element.argument.name;
                }
            } else if (element.type === 'ObjectProperty') {
                if (element.value.type === 'ObjectExpression') {
                    var chainProp = element.value.properties.find(function (item) {
                        return item.key.name === 'chain';
                    });
                    if (!chainProp) {
                        continue;
                    }
                    if (chainProp.value.name) {
                        part.type = 'chain';
                        part.identifier = chainProp.value.name;
                    } else if (chainProp.value.elements) {
                        var _subParts3 = parseCollection(chainProp.value.elements);
                        foundParts.push.apply(foundParts, _toConsumableArray(_subParts3));
                        continue;
                    }
                } else {
                    if (!element.value.name) {
                        continue;
                    }
                    part.type = 'chain_signal';
                    part.identifier = element.value.name;
                }
            } else if (element.type === 'ArrayExpression') {
                var _subParts4 = parseCollection(element.elements);
                foundParts.push.apply(foundParts, _toConsumableArray(_subParts4));
                continue;
            } else {
                console.log('Unknown element', element);
                continue;
            }

            foundParts.push(part);
            lastPart = part;
        }
    } catch (err) {
        _didIteratorError = true;
        _iteratorError = err;
    } finally {
        try {
            if (!_iteratorNormalCompletion && _iterator.return) {
                _iterator.return();
            }
        } finally {
            if (_didIteratorError) {
                throw _iteratorError;
            }
        }
    }

    return foundParts;
}